module TestWise
  class TestResult

    attr_accessor :test_file
    attr_accessor :test_case
    attr_accessor :started_at
    attr_accessor :finished_at
    attr_accessor :successful
    attr_accessor :error_trace
		attr_accessor :duration
		
    attr_accessor :tester_name
    attr_accessor :target_server
    attr_accessor :tester_machine

  end
end

class ReleasesController < ApplicationController

  before_filter :authorize, :except => [:index, :add, :create, :show, :edit, :update, :destroy, :manage, :add_execution, :show_execution, :edit_execution, :remove_issue_for_release, :remove_all_test_results, :remove_test_results, :post_test_results, :add_test_result, :new_build_execution] #TODO
  before_filter :find_optional_project, :only => [:add, :create]
  before_filter :find_project_by_project_id, :only => [:index]
  before_filter :find_release, :only => [:show, :edit, :update, :destroy, :add_execution, :edit_execution, :show_execution, :remove_issue_for_release, :remove_all_test_results, :remove_test_results, :post_test_results, :add_test_result]

  skip_before_filter :verify_authenticity_token

  # accept_key_auth :index, :show, :create, :add
  accept_api_auth :index, :show, :create, :add

  def index
    if @project
      @releases = @project.releases
    else
      @releases = Release.find(:all, :order => 'start_date')
    end
    respond_to do |format|
      format.html # index.html.erb
      format.xml { render :xml => @releases }
    end
  end

  def show
    @issue_matrix = IssueMatrix.new(@release.issues)
    if params[:execution_id]
      @execution = TestExecution.find_by_id(params[:execution_id])
    end
    @execution ||= @release.latest_execution

		@issues_priorities = []
		issue_priorties_groups = @release.issues.group_by{ |r| r.priority }
		issue_priorties_groups.each do |key, values|			
			status_count = values.size
			the_label = key.name
			@issues_priorities << [the_label, status_count]
		end
	end
	
  def edit
    @project = @release.project
  end
  
  def update
      respond_to do |format|
        if @release.update_attributes(params[:release])
          @release.refresh_issues(true)
          flash[:notice] = l(:notice_successful_update)
          format.html { redirect_to("/releases?project_id=#{@release.project.identifier}") }
          format.xml { head :ok }
        else
          format.html { render :action => "edit" }
          format.xml { render :xml => @release.errors, :status => :unprocessable_entity }
        end
      end    
  end

  # DELETE /releases/1
  # DELETE /releases/1.xml
  def destroy
    @release.destroy
    respond_to do |format|
      format.html { redirect_to("/releases?project_id=#{@release.project.identifier}") }
      format.xml { head :ok }
    end
  end

  def add
    @release = Release.new
    @release.project = @project
    render :layout => !request.xhr?
  end

  ## Create a new build and execution
  #
  #  http://localhost:3600/projects/pools/releases/new_build_execution?name=build_3&version=0.0.1&token=abc
  def new_build_execution()
    if params[:project_id]
      if params[:project_id] =~ /^\w/
        @project = Project.find_by_identifier(params[:project_id])
      else
        @project = Project.find(params[:project_id]) unless params[:project_id].blank?
      end
    end

    begin
      rel = Release.new
      rel.name = params[:name]
      the_version = @project.versions.find(:first, :conditions => ["name = ?", params[:version]])
      rel.version_id = the_version.id
      rel.build = params[:build]
      rel.project = @project

      closed_issues = @project.issues.select { |x| x.closed? }
      rel.issues << closed_issues
      rel.issues << rel.version.fixed_issues
      rel.save!
      
      the_exec = TestExecution.new(:token => params[:token], :started_at => Time.now)
      the_exec.release = rel
      the_exec.save

      render :text => the_exec.id
    rescue => e
      render :text => "Error: #{e}"
    end
    
  end

  def create
    puts "dEBUG release params: #{release_params.inspect}"
    @release = Release.new
    @release.assign_attributes(release_params, :without_protection => true) 
    @release.project ||= @project
    if @release.save
	    @release.refresh_issues(false)
      respond_to do |format|
        format.html {
          redirect_to(:action => 'show', :id => @release)
        }
        format.xml { render :action => 'show', :status => :created, :location => url_for(:controller => 'releases', :action => 'show', :id => @release) }
        format.json { render :text => @release.to_json, :status => :created, :location => url_for(:controller => 'releases', :action => 'show'), :layout => false }
      end
      return
    else
      respond_to do |format|
        format.html { render :action => 'add' }
        format.xml { render(:xml => @release.errors, :status => :unprocessable_entity); return }
        format.json { render :text => object_errors_to_json(@release), :status => :unprocessable_entity, :layout => false }
      end
    end
  end

  def add_execution
    @test_execution = TestExecution.new
    @test_execution.assign_attributes(params[:test_execution], :without_protection => true)     
    if request.post?
      @test_execution.release = @release
      if @test_execution.save
        redirect_to :action => "show", :id => @release.id
        return
      else
      end
    end
  end

  def add_test_result	
    @test_execution = TestExecution.find_by_id(params[:execution_id])
		@release = @test_execution.release
    @test_result = TestResult.new
    @test_result.assign_attributes(params[:test_result], :without_protection => true)     

    if request.post?
      @test_result.test_execution = @test_execution
      if @test_result.save
        redirect_to "/releases/show_execution?execution_id=#{@test_execution.id}"
        return
      end
    end
  end

  def edit_execution
    @test_execution = TestExecution.find_by_id(params[:execution_id])
		@release = @test_execution.release
  end

  def update_execution
    @test_execution = TestExecution.find_by_id(params[:execution_id])
		@release = @test_execution.release
      respond_to do |format|
        if @test_execution.update_attributes(params[:test_execution])
          flash[:notice] = l(:notice_successful_update)
          format.html { redirect_to :action => "show", :id => @release.id }
          format.xml { head :ok }
        else
          format.html { render :action => "edit_execution" }
          format.xml { render :xml => @test_execution.errors, :status => :unprocessable_entity }
        end
    end
  end

  def show_execution
    @test_execution = TestExecution.find_by_id(params[:execution_id])
		@release = @test_execution.release if @test_execution
  end

  # TODO
  def add_requirements
    @release.issues << @project.issues
    @release.save
  end

  def find_execution_for_testwise(release_name_or_id, token)
    @execution = TestExecution.find_by_token(token) # (:first, :conditions => ["token = ?", params[:token]])
    puts "[DEBUG] finding execution for token => |#{token}|  release => |#{release_name_or_id}| => #{@execution}"
    if @execution.release.name == release_name_or_id || @execution.release.id == release_name_or_id.to_i
      return @execution
    end
    return nil
  end

  def post_test_results
    if request.post?
      if params[:agent] =~ /TestWise/i
        @execution = find_execution_for_testwise(params[:id] || params[:release], params[:token])
        if @execution.nil? 
					render :text => "Error: No matching execution found: Release => #{params[:id] || params[:release]}, Token => #{params[:token]}"
        	return
				end
      elsif params[:agent] =~ /BuildWise/i      
        if params[:id] || params[:release] 
          @release =  Release.find_by_name(params[:release]) if params[:release]
          @release ||= Release.find_by_id(params[:id]) if params[:id] 
          @execution = TestExecution.new
          @execution.release = @release
          if TestExecution.last then
            @execution.token = "x" + (TestExecution.last.id + 1).to_s.rjust(3, "0")
          else
            @execution.token = "x001"
          end
          @execution.started_at = Time.now # TODO, probably shall be passed
          @execution.save				
        end
		  else
        render :text => "Error: Release/Execuction not found: #{params[:token]} of release #{params[:id]}"
        return
      end

      if @execution
        result_yaml = params[:result_yaml]

        puts "DEBUG: #{result_yaml}"
        results_obj = YAML.load(result_yaml)

        results_obj.each_with_index do |testwise_test_result, idx|
          a_test_result = @execution.test_results.find(:first, :conditions => ["test_file = ? AND test_name = ?", testwise_test_result.test_file, testwise_test_result.test_case])

          a_test_result ||= TestResult.new
          a_test_result.test_file = testwise_test_result.test_file
          a_test_result.test_name = testwise_test_result.test_case
          a_test_result.automated = true
          a_test_result.result = testwise_test_result.successful ? "passed" : "failed"

          a_test_result.target_server= testwise_test_result.target_server
          a_test_result.tester_name = testwise_test_result.tester_name

          if a_test_result.test_name =~ /\[(.*)\]/
            a_test_result.issues_covered = $1
          end

          a_test_result.tester_machine= testwise_test_result.tester_machine

          a_test_result.started_at = testwise_test_result.started_at
          a_test_result.finished_at = testwise_test_result.finished_at
					a_test_result.duration =  testwise_test_result.duration
          a_test_result.error_trace = testwise_test_result.error_trace
          a_test_result.save

          # puts a_test_result.errors.inspect

          @execution.started_at ||= testwise_test_result.started_at
          @execution.test_results << a_test_result
          @execution.save
        end
        render :text => "OK"
        return
      else
        puts "Failed to find execution"
        render :text => "Execution with token #{params[:token]} for release #{params[:release]} not found"
      end
    end
  end

  def remove_issue_for_release
    the_issue = Issue.find_by_id(params[:issue_id])
    @release.issues.delete(the_issue)
		@issue = the_issue
    respond_to do |format|
			format.html { redirect_to :action => :show, :id => @release.id }
			format.js { }
		end
		  
  end

  def remove_test_results
    @test_execution = TestExecution.find_by_id(params[:execution_id])
		@release = @test_execution.release
    @test_result = TestResult.find_by_id(params[:test_result_id])
    outcome = @test_execution.test_results.delete(@test_result)
    respond_to do |format|
			format.html { redirect_to "/releases/show_execution?execution_id=#{@test_execution.id}" }
			format.js { }
		end
		
    # render :partial => "execution_results", :test_execution  => @test_execution
    # redirect_to :action => :show_execution, :id => @release.id, :execution_id => @test_execution.id
  end

  def remove_all_test_results
    @test_execution = TestExecution.find_by_id(params[:execution_id])
    @test_execution.test_results.clear
		respond_to do |format|
    	format.js { }
		end 
    # redirect_to :action => :show_execution, :id => @release.id, :execution_id => @test_execution.id
  end

  private

  def release_params
    params.require("release").permit(:version_id, :name, :description)
  end

  def find_release
    begin
      @release = Release.find_by_id(params[:id])
      if @release.nil?
        @release = Release.find_by_id(params[:name])
      end
      @project = @release.project
    rescue => e
      puts "Error on finding release: #{params[:id]}"
    end
  end

end