class SpecificationsController < ApplicationController
  before_filter :find_specification, :except => [:find_specification, :show, :export_excel]

    # AgileWay
    #
    # Generate Requirements Traceability Matrix
    def traceability_matrix
      project_id = params[:id]
      @project = Project.find_by_id(project_id)
      allowed = User.current.allowed_to?({:controller => params[:controller], :action => "index" }, @project, :global => true)
      begin
        require 'spreadsheet'
        matrix_template_file = File.join(::Rails.root, 'config', 'traceability_matrix_template.xls')
        book = Spreadsheet.open matrix_template_file
        sheet = book.worksheet 0
        generated_at = Time.now.strftime('Date: %Y-%m-%d %H:%M:%S')
        sheet.row(2).replace [sheet.cell(2, 0), "", "", "", "", generated_at]
        sheet.row(4).replace ["Requirement test coverage:", $specification_stats[:coverage] * 1.0 / 100 ]
        sheet.row(5).replace ["User Story count:", @project.active_issues.count]
        sheet.row(6).replace ["Test script files count:", $specification_stats[:test_files].size]

        requirement_sheet = book.worksheet("Requirements")
        test_case_sheet = book.worksheet("Test Cases")
        # NOTES need to reference these two sheets, otherwise won't work,
        # puts test_case_sheet.inspect
        # puts requirement_sheet.inspect

        all_test_cases = []
        case_idx = 0
        $specification_stats[:spec_files].each do |spec_file|
          spec_file.spec_contexts.each do |context|
            context.spec_behaviours.each do |test_case|
              case_idx += 1
              test_case_sheet.row(case_idx).replace([case_idx, spec_file.file_name, test_case.name])
              all_test_cases << test_case
            end
          end
        end
        all_test_case_count = all_test_cases.size

        sheet.row(7).replace ["Test case count:", all_test_case_count]
        test_lookups = $specification_stats[:requirement_tests]

        row_headings = ["Test Cases / Requirment IDs"]
        # all_test_cases.each_with_index {|x, idx|  row_headings << "TC " + (idx+1).to_s }
        story_ids = []
        uncovered_row_cells = []
				@project.active_issues.each_with_index {|story, idx|
					 row_headings << story.traceability_id
					 requirement_sheet.row(idx + 1).replace([story.traceability_id, story.subject])
					 tests_for_this_story = test_lookups[story.traceability_id.to_s]
	         if tests_for_this_story.nil? || tests_for_this_story.empty?
		         # this story not covered
						  uncovered_row_cells << idx
					 end
					 story_ids << story.traceability_id.to_s
				}

        row = 1
        all_test_cases.each_with_index { |test_case, idx|
          results = ["TC " + (idx + 1).to_s + ": " + test_case.name] # propbably name
          covered = false

          story_ids.each do |story_id|
	          tests_for_this_story = test_lookups[story_id.to_s]
            if tests_for_this_story && tests_for_this_story.include?(test_case)
               results << "x"
               covered = true
            else
               results << ""
            end
          end
          # new_row = sheet.insert_row 11, results
          sheet.row(11 + idx).replace results
          # sheet.row(10+idx).set_format(0, format_covered) if covered
        }

				# format_uncovered = ::Spreadsheet::Format.new(:color => :red,:weight => :bold, :size => 9)
    #     format_covered = ::Spreadsheet::Format.new(:color => :green,:weight => :bold, :size => 9)
         new_row = sheet.row(9).replace row_headings
				# uncovered_row_cells.each do |cell_idx|
				# 	sheet.row(9).set_format(1+cell_idx,  format_uncovered)
    #   	end


         matrix_sheet_story_count = @project.active_issues.size
         matrix_sheet_column_count = sheet.column_count
         (matrix_sheet_story_count+1..matrix_sheet_column_count).each do |col|
           sheet.columns[col].hidden  =true
         end

         # @project.active_issues.each_with_index do |story, idx|
         #  # puts sheet.row(10)[1]
         #  sheet.row(10)[idx+1].value = ""
         # end

        require 'tempfile'
        temp_file = Tempfile.new('traceability_matrix_report.xls')
        book.write(temp_file.path)
        # TODO how to close file
        send_file temp_file.path,  :filename =>  "traceability_matrix_report.xls"
      rescue => e
        puts "\nError => #{e}\n "
        puts e.backtrace
        render :nothing => true
      end
    end


    def traceability_matrix_v2
      project_id = params[:id]
      @project = Project.find_by_id(project_id)
      allowed = User.current.allowed_to?({:controller => params[:controller], :action => "index" }, @project, :global => true)
      begin
        require 'spreadsheet'
        matrix_template_file = File.join(::Rails.root, 'config', 'traceability_matrix_template.xls')
        book = Spreadsheet.open matrix_template_file
        sheet = book.worksheet 0
        generated_at = Time.now.strftime('Date: %d/%m/%Y %H:%M:%S')
        sheet.row(2).replace [sheet.cell(2, 0), "", "", "", "", generated_at]
        sheet.row(4).replace ["Requirement test coverage:", $specification_stats[:coverage] * 1.0 / 100 ]
        sheet.row(5).replace ["User Story count:", @project.active_issues.count]
        sheet.row(6).replace ["Test script files count:", $specification_stats[:test_files].size]

        requirement_sheet = book.worksheet("Requirements")
        test_case_sheet = book.worksheet("Test Cases")
        # NOTES need to reference these two sheets, otherwise won't work,
        puts test_case_sheet.inspect
        puts requirement_sheet.inspect

        all_test_cases = []
        case_idx = 0
        $specification_stats[:spec_files].each do |spec_file|
          spec_file.spec_contexts.each do |context|
            context.spec_behaviours.each do |test_case|
              case_idx += 1
              test_case_sheet.row(case_idx).replace([case_idx, spec_file.file_name, test_case.name])
              all_test_cases << test_case
            end
          end
        end
        all_test_case_count = all_test_cases.size

        sheet.row(7).replace ["Test case count:", all_test_case_count]
        test_lookups = $specification_stats[:requirement_tests]

        row_headings = ["IDs"]
        # all_test_cases.each_with_index {|x, idx|  row_headings << "TC " + (idx+1).to_s }
        story_ids = []
        uncovered_row_cells = []
				@project.active_issues.reverse.each_with_index {|story, idx|
					 row_headings << "REQ\n" + (story.traceability_id).to_s
					 requirement_sheet.row(idx + 1).replace([story.traceability_id, story.subject])
					 tests_for_this_story = test_lookups[story.traceability_id]
	         if tests_for_this_story.nil? || tests_for_this_story.empty?
		         # this story not covered
						  uncovered_row_cells << idx
					 end
					 story_ids << story.traceability_id
				}

        row = 1
        format_covered = ::Spreadsheet::Format.new :color => :blue,:weight => :bold, :size => 12
        all_test_cases.each_with_index { |test_case, idx|
          results = ["TC " + (idx + 1).to_s + ": " + test_case.name] # propbably name
          covered = false

          story_ids.each do |story_id|
	          tests_for_this_story = test_lookups[story_id]
            if tests_for_this_story && tests_for_this_story.include?(test_case)
               results << " X "
               covered = true
            else
               results << " "
            end
          end
          # new_row = sheet.insert_row 11, results
          sheet.row(10 + idx).replace results
          # sheet.row(10+idx).set_format(0, format_covered) if covered
        }

				format_uncovered = ::Spreadsheet::Format.new(:color => :red,:weight => :bold, :size => 9)
        new_row = sheet.row(9).replace row_headings
				uncovered_row_cells.each do |cell_idx|
					sheet.row(9).set_format(1+cell_idx,  format_uncovered)
      	end

        require 'tempfile'
        temp_file = Tempfile.new('traceability_matrix_report.xls')
        book.write(temp_file.path)

        send_file temp_file.path,  :filename =>  "traceability_matrix_report.xls"
      rescue => e
        puts "\nError => \n " + e
        puts e.backtrace
        render :nothing => true
      end
    end

    # list requirement => tests
    def traceability_matrix_v1
      project_id = params[:id]
      @project = Project.find_by_id(project_id)
      allowed = User.current.allowed_to?({:controller => params[:controller], :action => "index" }, @project, :global => true)
      puts "Allowing : #{allowed}"
      begin
        require 'spreadsheet'
        matrix_template_file = File.join(Rails.root, 'config', 'traceability_template.xls')
        book = Spreadsheet.open matrix_template_file
        sheet = book.worksheet 0
        generated_at = Time.now.strftime('Date: %d/%m/%Y %H:%M:%S')
        sheet.row(2).replace [sheet.cell(2, 0), "", "", "", "", generated_at]
        sheet.row(4).replace [nil, "Requirement test coverage:", $specification_stats[:coverage] * 1.0 / 100 ]
        sheet.row(5).replace [nil, "User Story count:", @project.issues.count]
        sheet.row(6).replace [nil, "Test script files count:", $specification_stats[:test_files].size]

        all_test_case_count = 0
        $specification_stats[:spec_files].each do |spec_file|
          all_test_case_count += spec_file.test_case_count
        end
        sheet.row(7).replace [nil, "Test case count:", all_test_case_count]
        puts "DEBUG: #{$specification_stats[:coverage]}"

        test_lookups = $specification_stats[:requirement_tests]
        @project.issues.each { |story|
          category_name = story.category.name rescue nil
          status_name = story.status.name rescue nil

          tests_for_this_story = test_lookups[story.id]
          if tests_for_this_story
            test_files_name = tests_for_this_story.collect {|x| File.basename(x.file_path) }.join(", ")
            test_cases_name = tests_for_this_story.collect {|x| x.name}.join(";\n")
          else
            test_files_name  = nil
            test_cases_name  = nil
          end

          new_row = sheet.insert_row 10, [story.id, story.subject, category_name, status_name, test_files_name,
                                         test_cases_name]
        }

        tmp_dir = File.join(RAILS_ROOT, "tmp")
        FileUtils.mkdir_p(tmp_dir) unless File.exists?(tmp_dir)
        matrix_xls_file = File.join(tmp_dir, "traceability_matrix_report.xls")
        book.write(matrix_xls_file)

        send_file matrix_xls_file
      rescue => e
        puts e
        #render :nothing => true
      end
    end

  def refresh
    if specification_params.nil? || specification_params.empty?
      redirect_to "/projects/#{params[:id]}/issues"
      return
    end
    @specification ||= Specification.new(specification_params)
    @specification.project = @project
    @specification.directory = specification_params[:directory]
    @specification.scan_sub_dir = specification_params[:scan_sub_dir]
    @specification.traceability_id = specification_params[:traceability_id]
    @specification.local_work_dir = specification_params[:local_work_dir]
    @specification.pre_analyse_command = specification_params[:pre_analyse_command]
    @specification.save!

		if @specification.pre_analyse_command && !@specification.pre_analyse_command.blank? then
			current_dir = File.expand_path(".")
			begin
				puts "XXX => executing command | #{@specification.pre_analyse_command}"
				FileUtils.chdir(@specification.directory)
				`#{@specification.pre_analyse_command}`
			rescue => e1
				puts "Failed to execute command #{@specification.pre_analyse_command}, #{e1}"
			ensure
				FileUtils.chdir(current_dir)
			end
		end

    $specification_stats = parse_tests_in(@specification.directory,  {:local_work_dir => @specification.local_work_dir, :scan_sub_dir => @specification.scan_sub_dir, :traceability_id => @specification.traceability_id} )
    $specification_stats ||= {}
    $specification_stats[:updated_at] = Time.now
    $test_lookups = $specification_stats[:requirement_tests]

    if $specification_stats[:requirement_tests]
      # page["refreshed_at"].replace_html Time.now # .strftime("%d/%m/%y %M")
      #   page["requrement_covered_by_tests"].replace_html
      #   session[:return_cart].delete(@resource_item.id)
      begin
        requirements_covered_list = $specification_stats[:requirement_tests].keys.uniq
        count = 0
        @project.issues.each do |story|

          story_traceability_id = story.traceability_id(@specification.traceability_id).to_s
          count += 1 if requirements_covered_list.include?(story_traceability_id)
        end
        $specification_stats[:coverage] = count * 100 / @project.active_issues.size

        if $specification_stats[:coverage]  >= 100
          $specification_stats[:coverage]  = 100
        end
      rescue => e
        puts "Error on parsing : #{e}"
        $specification_stats[:coverage] = 0.0
      end
    elsif @specification.directory.nil? || !File.exists?(@specification.directory)
      $specification_stats[:error] = "#{@specification.directory} not exists"
    else
      $specification_stats[:error] = "No issue id found in test cases, please see above example."
    end

    puts "Coverage => #{$specification_stats[:coverage]}"
    if request.xhr?
			render
    else
      #      redirect_to "/projects/#{@project.}/issues"
      redirect_to :controller => "issues", :action => "index", :project_id => @project
    end

  end

  def destroy
    @specification.destroy
    redirect_to :controller => 'projects', :action => 'settings', :id => @project, :tab => 'specification'
  end

  def show
    @issue = Issue.find_by_id(params[:id])
		begin
	    @issue_tests = $test_lookups[@issue.traceability_id.to_s] rescue []
	    first_test_file_content = @issue_tests[0].lines.join("\n")
	    require 'coderay'
	    tokens = CodeRay.scan first_test_file_content, :ruby
	    page = tokens.html(:line_numbers => :inline, :wrap => :page, :css => :style)
		rescue => e
		end
  end

  private

  def find_specification
    @project = Project.find(params[:id] || params[:project_id] )
    @specification = @project.specification
  rescue ActiveRecord::RecordNotFound
    render_404
  rescue InvalidRevisionParam
    show_error_not_found
  end

  def parse_tests_in(test_dir, opts = { })
    test_files = []
    spec_files = []
    test_lookups = {}

    return  if test_dir.nil? || !File.exists?(test_dir)
    # raise "acceptance test folder #{test_dir} not found" unless  test_dir &&  File.exists?(test_dir)

    require 'find'
		puts "\n\n\nXXX => #{opts[:local_work_dir]} | #{opts[:scan_sub_dir]}"

    Find.find(test_dir) do |f|

			unless opts[:scan_sub_dir]
				if File.directory?(f) && File.expand_path(f) != File.expand_path(test_dir)
					Find.prune
				end
			end

      if f =~ /(_spec|_test|_rwebspec)\.rb$/
        test_files << f
        spec_file = SpecFile.new(f)
				if opts[:local_work_dir]
					relative_path = File.expand_path(f).gsub(File.expand_path(test_dir), "")
					spec_file.local_work_path = File.join(opts[:local_work_dir], relative_path)
				end
        spec_file.spec_contexts.each { |ctx|
          ctx.spec_behaviours.each { |test_case|
            next if test_case.nil? or test_case.name.nil? or test_case.name.empty?
            # p test_case.name
            if test_case.name =~ /^\s*['|"]?\[(.*)\]/
              puts $1
              test_case_id_str = $1
              test_case_ids = test_case_id_str.split(",").collect {|x| x.strip }

              test_case_ids.each do |tid|
                if test_lookups[tid].nil? then
                  test_lookups[tid] = []
                end
                test_lookups[tid] << test_case
              end
            end
          }
        }
         spec_files << spec_file
      end
    end

    # puts "debug: found tests :#{test_lookups.inspect}"
    return { :test_files => test_files, :spec_files => spec_files, :requirement_tests => test_lookups}
  end




  def normalize_behaviour_name(beh_name)
    beh_name[0, 1] = '"' if beh_name[0, 1] == "'"
    beh_name[-1, 1] = '"' if beh_name[-1, 1] == "'"
    beh_name
  end


  def specification_params
    if params && params[:specification]
      params.require("specification").permit(:directory, :scan_sub_dir, :traceability_id, :local_work_dir, :pre_analyse_command)
    else
      return []
    end
  end
end
