class TestExecution < ActiveRecord::Base
  belongs_to :release
  has_many :test_results, :dependent => :destroy

  TEST_RESULT_STATUSES = %w(  not_run passed faileded error other  )

  validates_uniqueness_of :token

  def test_case_count
    test_results.size
  end

  def story_test_result_lookups
    hash = {}

      test_results.each do |tr|
        next if tr[:issues_covered].nil?
        covered_story_ids = tr[:issues_covered].split(",")

        covered_story_ids.each do |story_id|
          if tr[:result] == "passed" then
            if hash[story_id] && hash[story_id] |= 0
              hash[story_id] *= 1
            else
              hash[story_id] = 1
            end
          elsif tr[:result] == "not_run" then
            hash[story_id] = 0
          else
            hash[story_id] = -1
          end
        end
      
    end
    
    puts hash.inspect
    
    return hash
  end

  def analyse
    result_matrix = {:not_run => test_results.size, :passed => 0, :failed => 0, :other => 0, :error => 0, :covered_story_ids => []}
    test_results.each do |tr|
      result_matrix[tr.result.to_sym] ||= 0
      result_matrix[tr.result.to_sym] += 1
      result_matrix[:not_run] -= 1 unless tr.result == "not_run"
      result_matrix[:covered_story_ids] << tr[:issues_covered].split(",") if tr.passed? && tr[:issues_covered]
    end

    begin
      result_matrix[:covered_story_ids].flatten!.uniq!
      result_matrix[:test_pass_rate] = result_matrix[:passed] * 100 / test_case_count
    rescue => e
      puts "[covered_storyes_iuds] failed to analyse :#{e}"
    end


    all_issues_for_release = release.issues
    issues_covered = result_matrix[:covered_story_ids]

    release_issues_ids = all_issues_for_release.collect { |x| x.id }
    covered_issues_ids = result_matrix[:covered_story_ids].collect { |x| x.to_i }
    uncovered_issues_ids = release_issues_ids - covered_issues_ids

    begin
      result_matrix[:requirement_coverage] = (release_issues_ids.size - uncovered_issues_ids.size) * 100 / release_issues_ids.size
      result_matrix[:requirement_coverage] = 100 if result_matrix[:requirement_coverage] > 100
    rescue => e
      result_matrix[:requirement_coverage]  = 0
    end

    result_matrix
  end
end
